#pragma once
#include <iostream>
#include <iterator>
#include <fstream>
#include <algorithm>
#include <string>

class FileMaster
{
public:
	FileMaster();
	~FileMaster();

	void createData(std::ofstream& outStream);
	void createData(std::ostream_iterator<int> it);


	template<class container>
	void loadData(container& cont, std::ifstream& inStream);

	template<class container>
	void loadData(container& cont, std::istream_iterator<int> inItBegin, std::istream_iterator<int> inItEnd);

	template<class container>
	void modify(container& cont);   

	template<class Iter>
	void modify(Iter begin, Iter end);

	void modify(std::istream_iterator<int> inItBegin, std::istream_iterator<int> inItEnd, std::ostream_iterator<int> outItBegin);


	template<class container>
	void outputResult(const container& cont, std::ofstream& outStream);

	template<class container>
	void outputResult(const container& cont, std::ostream_iterator<int> outItBegin);
};

template<class container>
inline void FileMaster::loadData(container& cont, std::ifstream& inStream) {
	typename container::value_type thisNum = 0;
	
	if (!inStream.is_open()) 
	    throw new std::runtime_error("File is not opened!");               //!!! Где тогда delete ? И зачем вообще new использовать?

	if (!cont.empty()) {
		cont.clear();
	}

	for (int i = 0; i < 100; i++) {
		inStream >> thisNum;
		cont.push_back(thisNum);
	}
}

template<class container>
inline void FileMaster::loadData(container& cont, std::istream_iterator<int> inItBegin, std::istream_iterator<int> inItEnd) {
	if (!cont.empty()) {
		cont.clear();
	}
	cont.resize(100);

	std::copy(inItBegin, inItEnd, cont.begin());
}

template<class container>
inline void FileMaster::modify(container& cont) {
	if (cont.empty()) throw new std::exception("Container is empty, nothing to modify!");

	int summ = 0;
	std::for_each(cont.begin(), cont.end(), [&summ](int current) {
		summ += current; });
	int avg = summ / cont.size();

	std::for_each(cont.begin(), cont.end(), [&avg](auto& element) {
		if (abs(element) % 2) {
			element = element / avg;
		}
		}
	);
}

template<class Iter>
inline void FileMaster::modify(Iter begin, Iter end) {
	if (begin == end) throw new std::exception("Container is empty, nothing to modify!");               //!!! Где тогда delete ? И зачем вообще new использовать?

	int summ = 0;
	unsigned count = 0;
	std::for_each(begin, end, [&summ, &count](int current) {
		summ += current;
		++count;  
		}
	);
	int avg = summ / count;

	std::for_each(begin, end, [&avg](auto& element) {
		if (abs(element) % 2) {
			element = element / avg;
		}
		}
	);
}

template<class container>
inline void FileMaster::outputResult(const container& cont, std::ofstream& outStream) {
	if (!outStream.is_open()) {
		throw std::runtime_error("File is not opened!");
	}
	if (cont.empty()) {
		throw std::exception("Container is empty, nothing to output!");
	}

	std::for_each(cont.begin(), cont.end(), [&outStream](int elem) {
		outStream << std::to_string(elem) << " "; });
}

template<class container>
inline void FileMaster::outputResult(const container& cont, std::ostream_iterator<int> outItBegin) {
	if (cont.empty()) {
		throw std::exception("Container is empty, nothing to output!");
	}
	std::copy(cont.begin(), cont.end(), outItBegin);
}